import pytest
import mock
from pullbug.cli import PullBug


@mock.patch('pullbug.cli.LOGGER')
def test_throw_missing_error(mock_logger):
    with pytest.raises(ValueError):
        PullBug.throw_missing_error('GITHUB_TOKEN')
    mock_logger.critical.assert_called_once_with(
        'No GITHUB_TOKEN set. Please correct and try again.'
    )


@mock.patch('pullbug.cli.LOGGER')
def test_run_missing_checks_no_github_or_gitlab(mock_logger):
    with pytest.raises(ValueError):
        PullBug.run_missing_checks(False, False, False, False)
    mock_logger.critical.assert_called_once_with(
        'Neither "github" nor "gitlab" flags were passed, one is required. Please correct and try again.'
    )


@mock.patch('pullbug.cli.GITHUB_TOKEN', '123')
@mock.patch('pullbug.cli.PullBug.run_missing_checks')
@mock.patch('pullbug.github_bug.GithubBug.run')
@mock.patch('pullbug.cli.LOGGER')
def test_run_github(mock_logger, mock_github, mock_missing_checks):
    PullBug.run(True, False, False, False, False, 'mock-owner', 'open', 'orgs', 'opened', 'all')
    mock_missing_checks.assert_called_once()
    mock_github.assert_called_once()
    mock_logger.info.assert_called()


@mock.patch('pullbug.cli.GITHUB_TOKEN', '123')
@mock.patch('pullbug.cli.PullBug.run_missing_checks')
@mock.patch('pullbug.gitlab_bug.GitlabBug.run')
@mock.patch('pullbug.cli.LOGGER')
def test_run_gitlab(mock_logger, mock_gitlab, mock_missing_checks):
    PullBug.run(False, True, False, False, False, 'mock-owner', 'open', 'orgs', 'opened', 'all')
    mock_missing_checks.assert_called_once()
    mock_gitlab.assert_called_once()
    mock_logger.info.assert_called()


@mock.patch('pullbug.cli.LOGGER')
def test_run_missing_checks_no_github_token(mock_logger):
    message = 'No GITHUB_TOKEN set. Please correct and try again.'
    with pytest.raises(ValueError):
        PullBug.run_missing_checks(True, False, False, False)
    mock_logger.critical.assert_called_once_with(message)


@mock.patch('pullbug.cli.LOGGER')
def test_run_missing_checks_no_gitlab_api_key(mock_logger):
    message = 'No GITLAB_API_KEY set. Please correct and try again.'
    with pytest.raises(ValueError):
        PullBug.run_missing_checks(False, True, False, False)
    mock_logger.critical.assert_called_once_with(message)


@mock.patch('pullbug.cli.GITHUB_TOKEN', '123')
@mock.patch('pullbug.cli.LOGGER')
def test_run_missing_checks_no_slack_bot_token(mock_logger):
    message = 'No SLACK_BOT_TOKEN set. Please correct and try again.'
    with pytest.raises(ValueError):
        PullBug.run_missing_checks(True, False, True, False)
    mock_logger.critical.assert_called_once_with(message)


@mock.patch('pullbug.cli.SLACK_BOT_TOKEN', '123')
@mock.patch('pullbug.cli.GITHUB_TOKEN', '123')
@mock.patch('pullbug.cli.LOGGER')
def test_run_missing_checks_no_slack_channel(mock_logger):
    message = 'No SLACK_CHANNEL set. Please correct and try again.'
    with pytest.raises(ValueError):
        PullBug.run_missing_checks(True, False, True, False)
    mock_logger.critical.assert_called_once_with(message)


@mock.patch('pullbug.cli.GITHUB_TOKEN', '123')
@mock.patch('pullbug.cli.LOGGER')
def test_run_missing_checks_no_rocket_chat_url(mock_logger):
    message = 'No ROCKET_CHAT_URL set. Please correct and try again.'
    with pytest.raises(ValueError):
        PullBug.run_missing_checks(True, False, False, True)
    mock_logger.critical.assert_called_once_with(message)


@mock.patch('pullbug.cli.ROCKET_CHAT_URL', 'http://mock-url.com')
@mock.patch('pullbug.cli.SLACK_CHANNEL', 'mock-channel')
@mock.patch('pullbug.cli.SLACK_BOT_TOKEN', '123')
@mock.patch('pullbug.cli.GITHUB_TOKEN', '123')
@mock.patch('pullbug.cli.LOGGER')
def test_run_missing_checks_no_errors(mock_logger):
    PullBug.run_missing_checks(True, False, False, True)
    mock_logger.critical.assert_not_called()



import os
import requests
import logging
from pullbug.logger import PullBugLogger
from pullbug.messages import Messages


GITLAB_API_KEY = os.getenv('GITLAB_API_KEY')
GITLAB_API_URL = os.getenv('GITLAB_API_URL', 'https://gitlab.com/api/v4')
IGNORE_WIP = os.getenv('IGNORE_WIP')
GITLAB_HEADERS = {
    'authorization': f'Bearer {GITLAB_API_KEY}'
}
LOGGER = logging.getLogger(__name__)


class GitlabBug():
    @classmethod
    def run(cls, gitlab_scope, gitlab_state, wip, slack, rocketchat):
        """Run the logic to get MR's from GitLab and
        send that data via message.
        """
        PullBugLogger._setup_logging(LOGGER)
        merge_requests = cls.get_merge_requests(gitlab_scope, gitlab_state)
        message_preamble = ''
        if merge_requests == []:
            message = 'No merge requests are available from GitLab.'
            LOGGER.info(message)
            return message
        message_preamble = '\n:bug: *The following merge requests on GitLab ar still open and need your help!*\n'
        merge_request_messages = cls.iterate_merge_requests(merge_requests, wip)
        final_message = message_preamble + merge_request_messages
        if slack:
            Messages.slack(final_message)
        if rocketchat:
            Messages.rocketchat(final_message)
        LOGGER.info(final_message)

    @classmethod
    def get_merge_requests(cls, gitlab_scope, gitlab_state):
        """Get all repos of the GITLAB_API_URL.
        """
        LOGGER.info('Bugging GitLab for merge requests...')
        try:
            response = requests.get(
                f"{GITLAB_API_URL}/merge_requests?scope={gitlab_scope}&state={gitlab_state}&per_page=100",
                headers=GITLAB_HEADERS
            )
            print(response.json())
            LOGGER.info('GitLab merge requests retrieved!')
            if 'does not have a valid value' in response.text:
                error = f'Could not retrieve GitLab merge requests due to bad parameter: {gitlab_scope} | {gitlab_state}.'  # noqa
                LOGGER.warning(error)
                raise ValueError(error)
        except requests.exceptions.RequestException as response_error:
            LOGGER.warning(
                f'Could not retrieve GitLab merge requests: {response_error}'
            )
            raise requests.exceptions.RequestException(response_error)
        return response.json()

    @classmethod
    def iterate_merge_requests(cls, merge_requests, wip):
        """Iterate through each merge request and send
        a message to Slack if a PR exists.
        """
        final_message = ''
        for merge_request in merge_requests:
            # TODO: There is a "work_in_progress" key in the response
            # that could be used? https://docs.gitlab.com/ee/api/merge_requests.html
            if not wip and 'WIP' in merge_request['title'].upper():
                continue
            else:
                message = cls.prepare_message(merge_request)
                final_message += message
        return final_message

    @classmethod
    def prepare_message(cls, merge_request):
        """Prepare the message with merge request data.
        """
        try:
            if merge_request['assignees'][0]['username']:
                users = ''
                for assignee in merge_request['assignees']:
                    user = f"<{assignee['web_url']}|{assignee['username']}>"
                    users += user + ' '
            else:
                users = 'No assignee'
        except IndexError:
            users = 'No assignee'

        # Truncate description after 120 characters
        description = (merge_request['description'][:120] +
                       '...') if len(merge_request['description']) > 120 else merge_request['description']
        message = f"\n:arrow_heading_up: *Merge Request:* <{merge_request['web_url']}|" + \
            f"{merge_request['title']}>\n*Description:* {description}\n*Waiting on:* {users}\n"

        return message



import os
import logging
import requests
import slack


SLACK_BOT_TOKEN = os.getenv('SLACK_BOT_TOKEN')
SLACK_CHANNEL = os.getenv('SLACK_CHANNEL')
ROCKET_CHAT_URL = os.getenv('ROCKET_CHAT_URL')
LOGGER = logging.getLogger(__name__)


class Messages():
    @classmethod
    def rocketchat(cls, message):
        """Send a Rocket Chat message
        """
        try:
            requests.post(ROCKET_CHAT_URL, data={'text': message})
            LOGGER.info('Rocket Chat message sent!')
        except requests.exceptions.RequestException as rc_error:
            LOGGER.warning(f'Could not send Rocket Chat message: {rc_error}')
            raise requests.exceptions.RequestException(rc_error)

    @classmethod
    def slack(cls, message):
        """Send Slack messages via a bot
        """
        slack_client = slack.WebClient(SLACK_BOT_TOKEN)
        try:
            slack_client.chat_postMessage(
                channel=SLACK_CHANNEL,
                text=message
            )
            LOGGER.info('Slack message sent!')
        except slack.errors.SlackApiError as slack_error:
            LOGGER.warning(f'Could not send Slack message: {slack_error}')
            raise slack.errors.SlackApiError(
                slack_error.response["ok"], slack_error.response['error']
            )



import argparse
import os
import logging
from dotenv import load_dotenv
from pullbug.logger import PullBugLogger
from pullbug.github_bug import GithubBug
from pullbug.gitlab_bug import GitlabBug


GITHUB_TOKEN = os.getenv('GITHUB_TOKEN')
GITLAB_API_KEY = os.getenv('GITLAB_API_KEY')
GITLAB_API_URL = os.getenv('GITLAB_API_URL', 'https://gitlab.com/api/v4')
SLACK_BOT_TOKEN = os.getenv('SLACK_BOT_TOKEN')
SLACK_CHANNEL = os.getenv('SLACK_CHANNEL')
ROCKET_CHAT_URL = os.getenv('ROCKET_CHAT_URL')
LOGGER = logging.getLogger(__name__)


class PullBugCLI():
    def __init__(self):
        """Initiate CLI args.
        """
        # TODO: Validate all these with a set list of options
        parser = argparse.ArgumentParser(
            description='Get bugged via Slack or RocketChat to merge your GitHub pull requests or GitLab merge requests.'  # noqa
        )
        parser.add_argument(
            '-gh',
            '--github',
            required=False,
            action='store_true',
            default=False,
            help='Get bugged about pull requests from GitHub.'
        )
        parser.add_argument(
            '-gl',
            '--gitlab',
            required=False,
            action='store_true',
            default=False,
            help='Get bugged about merge requests from GitLab.'
        )
        parser.add_argument(
            '-s',
            '--slack',
            required=False,
            action='store_true',
            default=False,
            help='Send Pullbug messages to Slack.'
        )
        parser.add_argument(
            '-rc',
            '--rocketchat',
            required=False,
            action='store_true',
            default=False,
            help='Send Pullbug messages to Rocket.Chat.'
        )
        parser.add_argument(
            '-w',
            '--wip',
            required=False,
            action='store_true',
            default=False,
            help='Include "Work in Progress" pull or merge requests.'
        )
        parser.add_argument(
            '-gho',
            '--github_owner',
            required=False,
            type=str,
            default=None,
            help='The GitHub owner to retrieve pull requests from (can be a user or org).'
        )
        parser.add_argument(
            '-ghs',
            '--github_state',
            required=False,
            type=str,
            default='open',
            help='The GitHub state to retrieve pull requests with. (Default: open | closed | all)'
        )
        parser.add_argument(
            '-ghc',
            '--github_context',
            required=False,
            type=str,
            default='orgs',
            help='The GitHub context to retrieve pull requests with (Default: orgs | users).'
        )
        parser.add_argument(
            '-glst',
            '--gitlab_state',
            required=False,
            type=str,
            default='opened',
            help='The GitLab state to retrieve merge requests with. (Default: opened | closed | locked | merged)'
        )
        parser.add_argument(
            '-glsc',
            '--gitlab_scope',
            required=False,
            type=str,
            default='all',
            help='The GitLab state to retrieve pull requests with. (Default: all | created_by_me | assigned_to_me)'
        )
        parser.parse_args(namespace=self)

    def run(self):
        """Send command line args to the main run function.
        """
        PullBug.run(
            github=self.github,
            gitlab=self.gitlab,
            slack=self.slack,
            rocketchat=self.rocketchat,
            wip=self.wip,
            github_owner=self.github_owner,
            github_state=self.github_state,
            github_context=self.github_context,
            gitlab_state=self.gitlab_state,
            gitlab_scope=self.gitlab_scope,
        )


class PullBug():
    @classmethod
    def run(cls, github, gitlab, slack, rocketchat, wip, github_owner,
            github_state, github_context, gitlab_state, gitlab_scope):
        """Run Pullbug based on the configuration.
        """
        PullBugLogger._setup_logging(LOGGER)
        LOGGER.info('Running Pullbug...')
        load_dotenv()
        cls.run_missing_checks()
        if github:
            GithubBug.run(github_owner, github_state, github_context, wip, slack, rocketchat)
        if gitlab:
            GitlabBug.run(gitlab_scope, gitlab_state, wip, slack, rocketchat)
        LOGGER.info('Pullbug finished bugging!')

    @classmethod
    def run_missing_checks(cls, github, gitlab, slack, rocketchat):
        """Check that values are set based on
        configuration before proceeding.
        """
        if not github and not gitlab:
            message = 'Neither "github" nor "gitlab" flags were passed, one is required. Please correct and try again.'
            LOGGER.critical(message)
            raise ValueError(message)
        if github and not GITHUB_TOKEN:
            cls.throw_missing_error('GITHUB_TOKEN')
        if gitlab and not GITLAB_API_KEY:
            cls.throw_missing_error('GITLAB_API_KEY')
        if slack and not SLACK_BOT_TOKEN:
            cls.throw_missing_error('SLACK_BOT_TOKEN')
        if slack and not SLACK_CHANNEL:
            cls.throw_missing_error('SLACK_CHANNEL')
        if rocketchat and not ROCKET_CHAT_URL:
            cls.throw_missing_error('ROCKET_CHAT_URL')

    @classmethod
    def throw_missing_error(cls, missing):
        """Raise an error based on what env variables
        are missing.
        """
        message = f'No {missing} set. Please correct and try again.'
        LOGGER.critical(message)
        raise ValueError(message)


if __name__ == '__main__':
    PullBugCLI().run()



import mock
from pullbug import PullBugLogger


@mock.patch('pullbug.logger.LOG_PATH', 'test/mock-dir')
@mock.patch('pullbug.logger.LOG_FILE', './test/test.log')
@mock.patch('os.makedirs')
@mock.patch('pullbug.github_bug.LOGGER')
def test_setup_logging(mock_logger, mock_make_dirs):
    with mock.patch('builtins.open', mock.mock_open()):
        PullBugLogger._setup_logging(mock_logger)
    mock_make_dirs.assert_called_once()
    mock_logger.setLevel.assert_called()
    mock_logger.addHandler.assert_called()


@mock.patch('os.makedirs')
@mock.patch('pullbug.github_bug.LOGGER')
def test_setup_logging_dir_exists(mock_logger, mock_make_dirs):
    with mock.patch('builtins.open', mock.mock_open()):
        PullBugLogger._setup_logging(mock_logger)
    mock_make_dirs.assert_not_called()
    mock_logger.setLevel.assert_called()
    mock_logger.addHandler.assert_called()



import mock
import pytest
import requests
from pullbug.github_bug import GithubBug


@pytest.fixture
def _mock_github_token():
    return '123'


@pytest.fixture
def _mock_user():
    return 'mock-user'


@pytest.fixture
def _mock_github_context():
    return 'users'


@pytest.fixture
def _mock_github_state():
    return 'open'


@pytest.fixture
def _mock_repo():
    mock_repo = {
        'name': 'mock-repo'
    }
    return mock_repo


@pytest.fixture
def _mock_pull_request(_mock_user, _mock_repo):
    mock_pull_request = {
        'title': 'mock-pull-request',
        'description': 'Mock description',
        'assignees': [
            {
                'login': _mock_user,
                'html_url': f'https://github.com/{_mock_user}'
            },
        ],
        'body': 'Mock body of a pull request.',
        'html_url': f'https://github.com/{_mock_user}/{_mock_repo}/pull/1'
    }
    return mock_pull_request


@mock.patch('pullbug.cli.GITHUB_TOKEN', '123')
@mock.patch('pullbug.github_bug.GithubBug.get_pull_requests')
@mock.patch('pullbug.github_bug.GithubBug.get_repos')
@mock.patch('pullbug.github_bug.LOGGER')
def test_run_success(mock_logger, mock_get_repos, mock_pull_request):
    GithubBug.run('mock-owner', 'open', 'orgs', False, False, False)
    mock_get_repos.assert_called_once()
    mock_pull_request.assert_called_once()
    mock_logger.info.assert_called()


@mock.patch('pullbug.cli.GITHUB_TOKEN', '123')
@mock.patch('pullbug.github_bug.GithubBug.iterate_pull_requests')
@mock.patch('pullbug.github_bug.GithubBug.get_pull_requests', return_value=[])
@mock.patch('pullbug.github_bug.GithubBug.get_repos')
@mock.patch('pullbug.github_bug.LOGGER')
def test_run_no_pull_requests(mock_logger, mock_get_repos, mock_pull_request, mock_iterate_pull_requests):
    GithubBug.run('mock-owner', 'open', 'orgs', False, False, False)
    mock_get_repos.assert_called_once()
    mock_iterate_pull_requests.assert_not_called()
    mock_logger.info.assert_called()


@mock.patch('pullbug.cli.GITHUB_TOKEN', '123')
@mock.patch('pullbug.messages.Messages.slack')
@mock.patch('pullbug.github_bug.GithubBug.get_pull_requests')
@mock.patch('pullbug.github_bug.GithubBug.get_repos')
@mock.patch('pullbug.github_bug.LOGGER')
def test_run_with_slack(mock_logger, mock_get_repos, mock_pull_request, mock_slack):
    GithubBug.run('mock-owner', 'open', 'orgs', False, True, False)
    mock_get_repos.assert_called_once()
    mock_pull_request.assert_called_once()
    mock_slack.assert_called_once()
    mock_logger.info.assert_called()


@mock.patch('pullbug.cli.GITHUB_TOKEN', '123')
@mock.patch('pullbug.messages.Messages.rocketchat')
@mock.patch('pullbug.github_bug.GithubBug.get_pull_requests')
@mock.patch('pullbug.github_bug.GithubBug.get_repos')
@mock.patch('pullbug.github_bug.LOGGER')
def test_run_with_rocketchat(mock_logger, mock_get_repos, mock_pull_request, mock_rocketchat):
    GithubBug.run('mock-owner', 'open', 'orgs', False, False, True)
    mock_get_repos.assert_called_once()
    mock_pull_request.assert_called_once()
    mock_rocketchat.assert_called_once()
    mock_logger.info.assert_called()


@mock.patch('pullbug.github_bug.GITHUB_TOKEN', _mock_github_token)
@mock.patch('pullbug.github_bug.GITHUB_HEADERS')
@mock.patch('pullbug.github_bug.LOGGER')
@mock.patch('requests.get')
def test_get_repos_success(mock_request, mock_logger, mock_headers, _mock_user, _mock_github_context):
    # TODO: Mock this request better and assert additional values
    GithubBug.get_repos(_mock_user, _mock_github_context)
    mock_request.assert_called_once_with(
        f'https://api.github.com/{_mock_github_context}/{_mock_user}/repos?per_page=100',
        headers=mock_headers
    )
    assert mock_logger.info.call_count == 2


@mock.patch('pullbug.github_bug.LOGGER')
@mock.patch('requests.get', side_effect=requests.exceptions.RequestException('mock-error'))
def test_get_repos_exception(mock_request, mock_logger, _mock_user, _mock_github_context):
    with pytest.raises(requests.exceptions.RequestException):
        GithubBug.get_repos(_mock_user, _mock_github_context)
    mock_logger.warning.assert_called_once_with(
        'Could not retrieve GitHub repos: mock-error'
    )


@mock.patch('pullbug.github_bug.GITHUB_TOKEN', _mock_github_token)
@mock.patch('pullbug.github_bug.GITHUB_HEADERS')
@mock.patch('pullbug.github_bug.LOGGER')
@mock.patch('requests.get')
def test_get_pull_requests_success(mock_request, mock_logger, mock_headers, _mock_repo, _mock_github_state):
    # TODO: Mock this request better and assert additional values
    mock_repos = [_mock_repo]
    result = GithubBug.get_pull_requests(mock_repos, _mock_user, _mock_github_state)
    mock_request.assert_called_once_with(
        f'https://api.github.com/repos/{_mock_user}/{_mock_repo["name"]}/pulls?state={_mock_github_state}&per_page=100',
        headers=mock_headers
    )
    assert mock_logger.info.call_count == 2
    assert isinstance(result, list)


@mock.patch('pullbug.github_bug.LOGGER')
@mock.patch('requests.get', side_effect=requests.exceptions.RequestException('mock-error'))
def test_get_pull_requests_request_exception(mock_request, mock_logger, _mock_repo):
    mock_repos = [_mock_repo]
    with pytest.raises(requests.exceptions.RequestException):
        GithubBug.get_pull_requests(mock_repos, _mock_user, _mock_github_state)
    mock_logger.warning.assert_called_once_with(
        f'Could not retrieve GitHub pull requests for {_mock_repo["name"]}: mock-error'
    )


@mock.patch('pullbug.github_bug.LOGGER')
@mock.patch('requests.get', side_effect=TypeError('mock-error'))
def test_get_pull_requests_type_error_exception(mock_request, mock_logger, _mock_repo):
    mock_repos = [_mock_repo]
    with pytest.raises(TypeError):
        GithubBug.get_pull_requests(mock_repos, _mock_user, _mock_github_state)
    mock_logger.warning.assert_called_once_with(
        f'Could not retrieve GitHub pull requests due to bad parameter: {_mock_user} | {_mock_github_state}.'
    )


@mock.patch('pullbug.github_bug.GithubBug.prepare_message')
def test_iterate_pull_requests_wip_title(mock_prepare_message, _mock_pull_request):
    _mock_pull_request['title'] = 'wip: mock-pull-request'
    mock_pull_requests = [_mock_pull_request]
    GithubBug.iterate_pull_requests(mock_pull_requests, True)
    mock_prepare_message.assert_called_once()


@mock.patch('pullbug.github_bug.GithubBug.prepare_message')
def test_iterate_pull_requests_wip_setting_absent(mock_prepare_message, _mock_pull_request):
    _mock_pull_request['title'] = 'wip: mock-pull-request'
    mock_pull_requests = [_mock_pull_request]
    GithubBug.iterate_pull_requests(mock_pull_requests, False)
    mock_prepare_message.assert_not_called()


def test_prepare_message(_mock_pull_request, _mock_user, _mock_repo):
    result = GithubBug.prepare_message(_mock_pull_request)
    assert 'Pull Request' in result
    assert f'{_mock_pull_request["assignees"][0]["html_url"]}|{_mock_pull_request["assignees"][0]["login"]}' in result
    assert f'{_mock_pull_request["html_url"]}|{_mock_pull_request["title"]}' in result


def test_prepare_message_no_assignees_data(_mock_pull_request):
    _mock_pull_request['assignees'][0]['login'] = None
    result = GithubBug.prepare_message(_mock_pull_request)
    assert '*Waiting on:* No assignee' in result


def test_prepare_message_no_assignee(_mock_pull_request):
    _mock_pull_request['assignees'] = []
    result = GithubBug.prepare_message(_mock_pull_request)
    assert '*Waiting on:* No assignee' in result



import logging
import logging.handlers
import os

PULLBUG_LOCATION = os.path.expanduser(
    os.getenv('PULLBUG_LOCATION', '~/pullbug')
)
LOG_PATH = os.path.join(PULLBUG_LOCATION, 'logs')
LOG_FILE = os.path.join(LOG_PATH, 'pullbug.log')


class PullBugLogger():
    @classmethod
    def _setup_logging(cls, logger):
        """Setup project logging (to console and log file).
        """
        if not os.path.exists(LOG_PATH):
            os.makedirs(LOG_PATH)
        logger.setLevel(logging.INFO)
        handler = logging.handlers.RotatingFileHandler(
            LOG_FILE,
            maxBytes=200000,
            backupCount=5
        )
        formatter = logging.Formatter(
            "%(asctime)s - %(levelname)s - %(message)s"
        )
        handler.setFormatter(formatter)
        logger.addHandler(logging.StreamHandler())
        logger.addHandler(handler)



import mock
import pytest
import requests
from pullbug.gitlab_bug import GitlabBug


@pytest.fixture
def _mock_user():
    mock_user = {
        'name': 'mock-user'
    }
    return mock_user


@pytest.fixture
def _mock_repo():
    mock_repo = {
        'name': 'mock-repo'
    }
    return mock_repo


@pytest.fixture
def _mock_url():
    return 'http://mock-url.com'


@pytest.fixture
def _mock_gitlab_scope():
    return 'all'


@pytest.fixture
def _mock_gitlab_state():
    return 'opened'


@pytest.fixture
def _mock_merge_request(_mock_user, _mock_repo, _mock_url):
    mock_merge_request = {
        'title': 'mock-pull-request',
        'description': 'Mock description',
        'assignees': [
            {
                'username': _mock_user,
                'web_url': f'https://github.com/{_mock_user}'
            }
        ],
        'body': 'Mock body of a pull request.',
        'web_url': f'{_mock_url}/{_mock_user}/{_mock_repo}/pull/1'
    }
    return mock_merge_request


@mock.patch('pullbug.gitlab_bug.GITLAB_API_KEY', '123')
@mock.patch('pullbug.gitlab_bug.GITLAB_API_URL', _mock_url)
@mock.patch('pullbug.messages.Messages.rocketchat')
@mock.patch('pullbug.messages.Messages.slack')
@mock.patch('pullbug.gitlab_bug.GitlabBug.prepare_message')
@mock.patch('pullbug.gitlab_bug.GitlabBug.iterate_merge_requests')
@mock.patch('pullbug.gitlab_bug.GitlabBug.get_merge_requests')
@mock.patch('pullbug.gitlab_bug.GITLAB_HEADERS')
@mock.patch('pullbug.gitlab_bug.LOGGER')
@mock.patch('requests.get')
def test_run_no_slack_or_rocketchat_messages(mock_request, mock_logger, mock_headers,
                                             mock_get_merge_requests, mock_iterate_merge_requests,
                                             mock_prepare_message, mock_slack, mock_rocketchat):
    GitlabBug.run(_mock_gitlab_scope, _mock_gitlab_state, False, False, False)
    mock_logger.info.assert_called()
    mock_get_merge_requests.assert_called_once()
    mock_iterate_merge_requests.assert_called()
    mock_slack.assert_not_called()
    mock_rocketchat.assert_not_called()


@mock.patch('pullbug.gitlab_bug.GITLAB_API_KEY', '123')
@mock.patch('pullbug.gitlab_bug.GITLAB_API_URL', _mock_url)
@mock.patch('pullbug.messages.Messages.rocketchat')
@mock.patch('pullbug.messages.Messages.slack')
@mock.patch('pullbug.gitlab_bug.GitlabBug.prepare_message')
@mock.patch('pullbug.gitlab_bug.GitlabBug.iterate_merge_requests')
@mock.patch('pullbug.gitlab_bug.GitlabBug.get_merge_requests')
@mock.patch('pullbug.gitlab_bug.GITLAB_HEADERS')
@mock.patch('pullbug.gitlab_bug.LOGGER')
@mock.patch('requests.get')
def test_run_slack_message(mock_request, mock_logger, mock_headers,
                           mock_get_merge_requests, mock_iterate_merge_requests,
                           mock_prepare_message, mock_slack, mock_rocketchat):
    GitlabBug.run(_mock_gitlab_scope, _mock_gitlab_state, False, True, False)
    mock_logger.info.assert_called()
    mock_get_merge_requests.assert_called_once()
    mock_iterate_merge_requests.assert_called()
    mock_slack.assert_called_once()
    mock_rocketchat.assert_not_called()


@mock.patch('pullbug.gitlab_bug.GITLAB_API_KEY', '123')
@mock.patch('pullbug.gitlab_bug.GITLAB_API_URL', _mock_url)
@mock.patch('pullbug.messages.Messages.rocketchat')
@mock.patch('pullbug.messages.Messages.slack')
@mock.patch('pullbug.gitlab_bug.GitlabBug.prepare_message')
@mock.patch('pullbug.gitlab_bug.GitlabBug.iterate_merge_requests')
@mock.patch('pullbug.gitlab_bug.GitlabBug.get_merge_requests')
@mock.patch('pullbug.gitlab_bug.GITLAB_HEADERS')
@mock.patch('pullbug.gitlab_bug.LOGGER')
@mock.patch('requests.get')
def test_run_rocketchat_message(mock_request, mock_logger, mock_headers,
                                mock_get_merge_requests, mock_iterate_merge_requests,
                                mock_prepare_message, mock_slack, mock_rocketchat):
    GitlabBug.run(_mock_gitlab_scope, _mock_gitlab_state, False, False, True)
    mock_logger.info.assert_called()
    mock_get_merge_requests.assert_called_once()
    mock_iterate_merge_requests.assert_called()
    mock_slack.assert_not_called()
    mock_rocketchat.assert_called_once()


@mock.patch('pullbug.gitlab_bug.GITLAB_API_KEY', '123')
@mock.patch('pullbug.gitlab_bug.GITLAB_API_URL', _mock_url)
@mock.patch('pullbug.messages.Messages.rocketchat')
@mock.patch('pullbug.messages.Messages.slack')
@mock.patch('pullbug.gitlab_bug.GitlabBug.prepare_message')
@mock.patch('pullbug.gitlab_bug.GitlabBug.iterate_merge_requests')
@mock.patch('pullbug.gitlab_bug.GitlabBug.get_merge_requests', return_value=[])
@mock.patch('pullbug.gitlab_bug.GITLAB_HEADERS')
@mock.patch('pullbug.gitlab_bug.LOGGER')
@mock.patch('requests.get')
def test_run_no_returned_merge_requests(mock_request, mock_logger, mock_headers,
                                        mock_get_merge_requests, mock_iterate_merge_requests,
                                        mock_prepare_message, mock_slack, mock_rocketchat):
    GitlabBug.run(_mock_gitlab_scope, _mock_gitlab_state, False, False, False)
    mock_logger.info.assert_called_with('No merge requests are available from GitLab.')
    mock_iterate_merge_requests.assert_not_called()
    mock_slack.assert_not_called()
    mock_rocketchat.assert_not_called()


@ mock.patch('pullbug.gitlab_bug.GITLAB_API_KEY', '123')
@ mock.patch('pullbug.gitlab_bug.GITLAB_API_URL', _mock_url)
@ mock.patch('pullbug.gitlab_bug.GITLAB_HEADERS')
@ mock.patch('pullbug.gitlab_bug.LOGGER')
@ mock.patch('requests.get')
def test_get_merge_requests_success(mock_request, mock_logger, mock_headers):
    # TODO: Mock this request better and assert additional values
    GitlabBug.get_merge_requests(_mock_gitlab_scope, _mock_gitlab_state)
    mock_request.assert_called_once_with(
        f'{_mock_url}/merge_requests?scope={_mock_gitlab_scope}&state={_mock_gitlab_state}&per_page=100',
        headers=mock_headers
    )
    assert mock_logger.info.call_count == 2


@ mock.patch('pullbug.gitlab_bug.LOGGER')
@ mock.patch('requests.get', side_effect=requests.exceptions.RequestException('mock-error'))
def test_get_repos_exception(mock_request, mock_logger):
    with pytest.raises(requests.exceptions.RequestException):
        GitlabBug.get_merge_requests(_mock_gitlab_scope, _mock_gitlab_state)
    mock_logger.warning.assert_called_once_with(
        'Could not retrieve GitLab merge requests: mock-error'
    )


@ mock.patch('pullbug.gitlab_bug.GitlabBug.prepare_message')
def test_iterate_merge_requests_wip_title(mock_prepare_message, _mock_merge_request):
    _mock_merge_request['title'] = 'wip: mock-merge-request'
    mock_merge_requests = [_mock_merge_request]
    GitlabBug.iterate_merge_requests(mock_merge_requests, True)
    mock_prepare_message.assert_called_once()


@ mock.patch('pullbug.gitlab_bug.GitlabBug.prepare_message')
def test_iterate_merge_requests_wip_setting_absent(mock_prepare_message, _mock_merge_request):
    _mock_merge_request['title'] = 'wip: mock-merge-request'
    mock_merge_requests = [_mock_merge_request]
    GitlabBug.iterate_merge_requests(mock_merge_requests, False)
    mock_prepare_message.assert_not_called()


def test_prepare_message(_mock_merge_request, _mock_url, _mock_user, _mock_repo):
    result = GitlabBug.prepare_message(_mock_merge_request)
    assert 'Merge Request' in result
    assert f'{_mock_merge_request["assignees"][0]["web_url"]}|{_mock_merge_request["assignees"][0]["username"]}' in result  # noqa
    assert f'{_mock_merge_request["web_url"]}|{_mock_merge_request["title"]}' in result


def test_prepare_message_no_assignees_data(_mock_merge_request):
    _mock_merge_request['assignees'][0]['username'] = None
    result = GitlabBug.prepare_message(_mock_merge_request)
    assert '*Waiting on:* No assignee' in result


def test_prepare_message_no_assignee(_mock_merge_request):
    _mock_merge_request['assignees'] = []
    result = GitlabBug.prepare_message(_mock_merge_request)
    assert '*Waiting on:* No assignee' in result



import mock
import pytest
import requests
import slack
from pullbug.messages import Messages


@mock.patch('pullbug.messages.ROCKET_CHAT_URL', 'http://mock-url.com')
@mock.patch('pullbug.messages.LOGGER')
@mock.patch('requests.post')
def test_rocket_chat_success(mock_request, mock_logger):
    message = 'mock message'
    Messages.rocketchat(message)
    mock_request.assert_called_once_with(
        'http://mock-url.com', data={'text': message}
    )
    mock_logger.info.assert_called_once_with('Rocket Chat message sent!')


@mock.patch('pullbug.messages.LOGGER')
@mock.patch('requests.post', side_effect=requests.exceptions.RequestException('mock-error'))
def test_rocket_chat_exception(mock_request, mock_logger):
    message = 'mock message'
    with pytest.raises(requests.exceptions.RequestException):
        Messages.rocketchat(message)
    mock_logger.warning.assert_called_once_with(
        'Could not send Rocket Chat message: mock-error'
    )


@mock.patch('pullbug.messages.SLACK_CHANNEL', 'mock-channel')
@mock.patch('pullbug.messages.SLACK_BOT_TOKEN', '123')
@mock.patch('pullbug.messages.LOGGER')
@mock.patch('slack.WebClient.chat_postMessage')
def test_slack_success(mock_slack, mock_logger):
    message = 'mock message'
    Messages.slack(message)
    mock_slack.assert_called_once_with(channel='mock-channel', text=message)
    mock_logger.info.assert_called_once_with('Slack message sent!')


@mock.patch('pullbug.messages.LOGGER')
@mock.patch('slack.WebClient.chat_postMessage',
            side_effect=slack.errors.SlackApiError(
                message='The request to the Slack API failed.',
                response={'ok': False, 'error': 'not_authed'}
            ))
def test_slack_exception(mock_slack, mock_logger):
    message = 'mock message'
    with pytest.raises(slack.errors.SlackApiError):
        Messages.slack(message)
    mock_logger.warning.assert_called_once_with(
        "Could not send Slack message: The request to the Slack API failed.\nThe server responded with: {'ok': False, 'error': 'not_authed'}"  # noqa
    )



import os
import requests
import logging
from pullbug.logger import PullBugLogger
from pullbug.messages import Messages

GITHUB_TOKEN = os.getenv('GITHUB_TOKEN')
GITHUB_OWNER = os.getenv('GITHUB_OWNER')
GITHUB_HEADERS = {
    'Authorization': f'token {GITHUB_TOKEN}',
    'Content-Type': 'application/json; charset=utf-8'
}
LOGGER = logging.getLogger(__name__)


class GithubBug():
    @classmethod
    def run(cls, github_owner, github_state, github_context, wip, slack, rocketchat):
        """Run the logic to get PR's from GitHub and
        send that data via message.
        """
        PullBugLogger._setup_logging(LOGGER)
        repos = cls.get_repos(github_owner, github_context)
        pull_requests = cls.get_pull_requests(repos, github_owner, github_state)
        message_preamble = ''
        if pull_requests == []:
            message = 'No pull requests are available from GitHub.'
            LOGGER.info(message)
            return message
        message_preamble = '\n:bug: *The following pull requests on GitHub are still open and need your help!*\n'
        pull_request_messages = cls.iterate_pull_requests(pull_requests, wip)
        final_message = message_preamble + pull_request_messages
        if slack:
            Messages.slack(final_message)
        if rocketchat:
            Messages.rocketchat(final_message)
        LOGGER.info(final_message)

    @classmethod
    def get_repos(cls, github_owner, github_context=''):
        """Get all repos of the GITHUB_OWNER.
        """
        LOGGER.info('Bugging GitHub for repos...')
        try:
            repos_response = requests.get(
                f'https://api.github.com/{github_context}/{github_owner}/repos?per_page=100',
                headers=GITHUB_HEADERS
            )
            LOGGER.debug(repos_response.text)
            if 'Not Found' in repos_response.text:
                error = f'Could not retrieve GitHub repos due to bad parameter: {github_owner} | {github_context}.'
                LOGGER.warning(error)
                raise ValueError(error)
            LOGGER.info('GitHub repos retrieved!')
        except requests.exceptions.RequestException as response_error:
            LOGGER.warning(
                f'Could not retrieve GitHub repos: {response_error}'
            )
            raise requests.exceptions.RequestException(response_error)
        return repos_response.json()

    @classmethod
    def get_pull_requests(cls, repos, github_owner, github_state):
        """Grab all pull requests from each repo.
        """
        LOGGER.info('Bugging GitHub for pull requests...')
        pull_requests = []
        for repo in repos:
            try:
                pull_response = requests.get(
                    f'https://api.github.com/repos/{github_owner}/{repo["name"]}/pulls?state={github_state}&per_page=100',  # noqa
                    headers=GITHUB_HEADERS
                )
                LOGGER.debug(pull_response.text)
                if pull_response.json():
                    for single_pull_request in pull_response.json():
                        pull_requests.append(single_pull_request)
                else:
                    continue
            except requests.exceptions.RequestException as response_error:
                LOGGER.warning(
                    f'Could not retrieve GitHub pull requests for {repo["name"]}: {response_error}'
                )
                raise requests.exceptions.RequestException(response_error)
            except TypeError:
                error = f'Could not retrieve GitHub pull requests due to bad parameter: {github_owner} | {github_state}.'  # noqa
                LOGGER.warning(error)
                raise TypeError(error)
        LOGGER.info('Pull requests retrieved!')
        return pull_requests

    @classmethod
    def iterate_pull_requests(cls, pull_requests, wip):
        """Iterate through each pull request of a repo
        and send a message to Slack if a PR exists.
        """
        final_message = ''
        for pull_request in pull_requests:
            if not wip and 'WIP' in pull_request['title'].upper():
                continue
            else:
                message = cls.prepare_message(pull_request)
                final_message += message
        return final_message

    @classmethod
    def prepare_message(cls, pull_request):
        """Prepare the message with pull request data.
        """
        # TODO: Check requested_reviewers array also
        try:
            if pull_request['assignees'][0]['login']:
                users = ''
                for assignee in pull_request['assignees']:
                    user = f"<{assignee['html_url']}|{assignee['login']}>"
                    users += user + ' '
            else:
                users = 'No assignee'
        except IndexError:
            users = 'No assignee'

        # Truncate description after 120 characters
        description = (pull_request['body'][:120] + '...') if len(pull_request
                                                                  ['body']) > 120 else pull_request['body']
        message = f"\n:arrow_heading_up: *Pull Request:* <{pull_request['html_url']}|" + \
            f"{pull_request['title']}>\n*Description:* {description}\n*Waiting on:* {users}\n"

        return message



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

REQUIREMENTS = [
    'requests >= 1.0.0',
    'slackclient >= 2.0.1',
    'python-dotenv >= 0.10.0'
]

setuptools.setup(
    name='pullbug',
    version='2.0.2',
    description='Get bugged via Slack or RocketChat to merge your GitHub pull requests or GitLab merge requests.',  # noqa
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='http://github.com/justintime50/pull-bug',
    author='Justintime50',
    license='MIT',
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=REQUIREMENTS,
    extras_require={
        'dev': [
            'pytest >= 6.0.0',
            'pytest-cov >= 2.10.0',
            'coveralls >= 2.1.2',
            'flake8 >= 3.8.0',
            'mock >= 4.0.0',
        ]
    },
    entry_points={
        'console_scripts': [
            'pullbug=pullbug.cli:main'
        ]
    },
    python_requires='>=3.6',
)
